/*
 *	9-bit serial port driver for Oxford 16550 UARTs.
 */

#include <linux/module.h>
#include <linux/config.h>
#include <linux/types.h>
#include <linux/kernel.h>
#include <linux/fs.h>
#include <linux/mm.h>
#include <linux/miscdevice.h>
#include <linux/reboot.h>
#include <linux/init.h>
#include <asm/uaccess.h>
#include <asm/hardware.h>
#include <asm/bitops.h>


static int serial9bit_users;
static int serial9bit_base;

static int serial9bit_open(struct inode *inode, struct file *file)
{
	if(test_and_set_bit(1,&serial9bit_users))
		return -EBUSY;
	MOD_INC_USE_COUNT;
	return 0;
}

static int serial9bit_release(struct inode *inode, struct file *file)
{
	serial9bit_users = 0;
	MOD_DEC_USE_COUNT;
	return 0;
}

static ssize_t serial9bit_write(struct file *file, const char *data, size_t len, loff_t *ppos)
{
	printk(KERN_INFO "Serial 9-bit driver write\n");
	/*  Can't seek (pwrite) on this device  */
	if (ppos != &file->f_pos)
		return -ESPIPE;
	
	/* Can't write odd number of bytes. (16-bit data expected) */
	if (len & 0x1)
		return -EINVAL;

	return 0;
}

static ssize_t serial9bit_read(struct file *filp, char * buf, size_t size, loff_t *offp)
{
	printk(KERN_INFO "Serial 9-bit driver read\n");
	/* Can't read odd number of bytes. (16-bit data) */
	if (size & 0x1)
		return -EINVAL;

	return 0;
}


static int serial9bit_ioctl(struct inode *inode, struct file *file,
	unsigned int cmd, unsigned long arg)
{
	switch(cmd){
	default:
		return -ENOIOCTLCMD;
	}
}

static struct file_operations serial9bit_fops=
{
	owner:		THIS_MODULE,
	write:		serial9bit_write,
	read:		serial9bit_read,
	ioctl:		serial9bit_ioctl,
	open:		serial9bit_open,
	release:	serial9bit_release,
};

static struct miscdevice serial9bit_miscdev=
{
	MISC_DYNAMIC_MINOR,
	"serial9bit",
	&serial9bit_fops
};

static int __init serial9bit_init(void)
{
	int ret;
	printk(KERN_INFO "Serial 9-bit driver init\n");

	ret = misc_register(&serial9bit_miscdev);

	if (ret)
		return ret;

	return 0;
}

static void __exit serial9bit_exit(void)
{
	misc_deregister(&serial9bit_miscdev);
}

module_init(serial9bit_init);
module_exit(serial9bit_exit);
